Attribute VB_Name = "SmadDefs"
'*************************************************************************************
' VB INCLUDE MODULE FOR APPLICATIONS USING SENSORAY SMART A/D BOARDS WITH SMARTAD.DLL.
'*************************************************************************************

Option Explicit

' Board-independent functions ---------------------------
Declare Sub DRV_InetNtoA Lib "SmartAD" Alias "InetNtoA" (ByVal DotAdrs As String, ByVal HostAdrs As Long)
Declare Function DRV_InetAtoN Lib "SmartAD" Alias "InetAtoN" (ByVal DotAdrs As String) As Long

' Board-dependent functions -----------------------------
Declare Function DRV_Calibrate Lib "SmartAD" Alias "Calibrate" (ByVal hBD As Long, ByVal ChNum As Long, ByVal Range As Long, ByVal Value As Double) As Double
Declare Function DRV_GetAmbientTemp Lib "SmartAD" Alias "GetAmbientTemp" (ByVal hBD As Long, ByVal ChGroup As Long) As Double
Declare Function DRV_GetBoardAdrs Lib "SmartAD" Alias "GetBoardAdrs" (ByVal hBD As Long) As Long
Declare Function DRV_GetFaultFlags Lib "SmartAD" Alias "GetFaultFlags" (ByVal hBD As Long) As Long
Declare Function DRV_GetFWVersion Lib "SmartAD" Alias "GetFWVersion" (ByVal hBD As Long) As Long
Declare Function DRV_GetProductID Lib "SmartAD" Alias "GetProductID" (ByVal hBD As Long) As Long
Declare Function DRV_GetSensorData Lib "SmartAD" Alias "GetSensorData" (ByVal hBD As Long, ByVal Chan As Long) As Double
Declare Function DRV_GetStatusFlags Lib "SmartAD" Alias "GetStatusFlags" (ByVal hBD As Long) As Long
Declare Function DRV_SetBoardAttr Lib "SmartAD" Alias "SetBoardAttr" (ByVal hBD As Long, ByVal ProductID As Long, ByVal ADRS As Long) As Long

Declare Sub DRV_GetAllSensors Lib "SmartAD" Alias "GetAllSensors" (ByVal hBD As Long, SensorData As Any)
Declare Sub DRV_GetGageCal Lib "SmartAD" Alias "GetGageCal" (ByVal hBD As Long, ByVal ChNum As Long, ByRef Slope As Double, ByRef Offset As Integer)
Declare Sub DRV_ResetBoard Lib "SmartAD" Alias "ResetBoard" (ByVal hBD As Long)
Declare Sub DRV_SetCoefficients Lib "SmartAD" Alias "SetCoefficients" (ByVal hBD As Long, ByVal ChNum As Long, ByVal c1 As Double, ByVal c2 As Double, ByVal c3 As Double)
Declare Sub DRV_SetFailMode Lib "SmartAD" Alias "SetFailMode" (ByVal hBD As Long, ByVal ChNum As Long, ByVal Enable As Long)
Declare Sub DRV_SetFilter Lib "SmartAD" Alias "SetFilter" (ByVal hBD As Long, ByVal ChNum As Long, ByVal val As Long)
Declare Sub DRV_SetGageCal Lib "SmartAD" Alias "SetGageCal" (ByVal hBD As Long, ByVal ChNum As Long, ByVal Slope As Double, ByVal Offset As Integer)
Declare Sub DRV_SetGageSpan Lib "SmartAD" Alias "SetGageSpan" (ByVal hBD As Long, ByVal ChNum As Long, ByVal Value As Long)
Declare Sub DRV_SetGageTare Lib "SmartAD" Alias "SetGageTare" (ByVal hBD As Long, ByVal ChNum As Long)
Declare Sub DRV_SetGageZero Lib "SmartAD" Alias "SetGageZero" (ByVal hBD As Long, ByVal ChNum As Long)
Declare Sub DRV_SetHiSpeedMode Lib "SmartAD" Alias "SetHiSpeedMode" (ByVal hBD As Long)
Declare Sub DRV_SetLoPwrStandby Lib "SmartAD" Alias "SetLoPwrStandby" (ByVal hBD As Long, ByVal Standby As Long)
Declare Sub DRV_SetReject50Hz Lib "SmartAD" Alias "SetReject50Hz" (ByVal hBD As Long)
Declare Sub DRV_SetSensorType Lib "SmartAD" Alias "SetSensorType" (ByVal hBD As Long, ByVal Chan As Long, ByVal SDC As LogSDC)


' Error codes returned by GetFaultFlags() ===============================================

Public Const FAULT_DAV = &H1&       ' Smart A/D DAV handshake fault detected.
Public Const FAULT_CRMT = &H2&      ' Smart A/D CRMT handshake fault detected.
Public Const FAULT_LED = &H4&       ' Fault LED never turns on or is turned on all the time.
Public Const FAULT_OPEN = &H8&      ' Can't open Smart A/D driver.
Public Const FAULT_REG = &H10&      ' Can't register Smart A/D board with driver.
Public Const FAULT_SIG = &H20&      ' Driver detected incorrect Smart A/D signature.
Public Const FAULT_CREATE = &H40&   ' Driver can't create board object.

' Sensor type codes used by SetSensorType() =============================================

Public Enum LogSDC
                                    'Voltage Input Ranges:
    TYP_VOLTS_10V = 1&              '  10V
    TYP_VOLTS_5V = 3&               '  5V
    TYP_VOLTS_500MV = 5&            '  500mV
    TYP_VOLTS_100MV = 6&            '  100mV
                                    'Thermocouples:
    TYP_TC_B = 34&                  '  B
    TYP_TC_C = 33&                  '  C
    TYP_TC_E = 8&                   '  E
    TYP_TC_J = 10&                  '  J
    TYP_TC_K = 12&                  '  K
    TYP_TC_N = 32&                  '  N
    TYP_TC_R = 18&                  '  R
    TYP_TC_S = 16&                  '  S
    TYP_TC_T = 14&                  '  T
                                    'RTDs:
    TYP_RTD_CU_10_1 = 41&           '  10 Ohm copper
    TYP_RTD_NI_120_1 = 35&          '  120 Ohm nickel
    TYP_RTD_NI_200_1 = 36&          '  200 Ohm nickel
    TYP_RTD_NI_1000_1 = 37&         '  1K Ohm nickel (1353 Ohms @ 60 DegC)
    TYP_RTD_NI_1000_2 = 42&         '  1K Ohm nickel (1285 Ohms @ 60 DegC)
    TYP_RTD_PT_100_385 = 20&        '  385 Ohm platinum (0.1C res, full sensor range)
    TYP_RTD_PT_100_385_HIRES = 38&  '  385 Ohm platinum (0.0125C res, range -200:+400C)
    TYP_RTD_PT_100_392 = 22&        '  392 Ohm platinum (0.1C res, full sensor range)
    TYP_RTD_PT_100_392_HIRES = 39&  '  392 Ohm platinum (0.0125C res, range -200:+400C)
    TYP_RTD_PT_1000_1 = 43&         '  1K Ohm platinum (1422.9 Ohms @ 110 DegC)
                                    'Thermistor:
    TYP_THERMISTOR_10K = 24&        '  10K with autorange
    TYP_THERMISTOR_10KFIX = 40&     '  10K on fixed 600K ohm range
                                    'Current Loop:
    TYP_CURLOOP = 25&               '  4-20 mA
                                    'Gage:
    TYP_GAGE = 26&                  '  Strain/pressure gage
                                    'Resistance:
    TYP_OHMS_400 = 27&              '  400 ohm range
    TYP_OHMS_3K = 28&               '  3K ohm range
    TYP_OHMS_600K = 29&             '  600K ohm range
    TYP_OHMS_CUSTOM = 30&           '  Custom resistive sensor
                                    'Disabled:
    TYP_DISABLED = 31&              '  Remove channel from scan list
End Enum

'************************************************************************************
' General-purpose support functions for Sensoray Smart A/D demo programs.
'************************************************************************************

Public Function FindAllPciBoards() As Long
    ' Find and register all PCI/CPCI Smart A/D boards in the system.
    ' Return the number of boards found.
    Dim NUMBOARDS As Long
    For NUMBOARDS = 0 To 15
        If DRV_SetBoardAttr(NUMBOARDS, 618, 0) = 0 Then
            If DRV_SetBoardAttr(NUMBOARDS, 619, 0) = 0 Then
                If DRV_SetBoardAttr(NUMBOARDS, 718, 0) = 0 Then
                    If DRV_SetBoardAttr(NUMBOARDS, 719, 0) = 0 Then
                        Exit For
                    End If
                End If
            End If
        End If
    Next NUMBOARDS
    FindAllPciBoards = NUMBOARDS
End Function

Public Function FaultMessage(BdNum As Long, Optional FaultFlags As Long = -1) As String
    ' Construct fault message based on fault flags.
    Dim Msg     As String
    ' Fetch fault flags from board if requested by omission of parameter.
    If FaultFlags = -1 Then FaultFlags = DRV_GetFaultFlags(BdNum)
    ' Construct message.
    If FaultFlags Then
        If (FaultFlags And FAULT_CREATE) Then
            Msg = "  Can't create object"
        ElseIf (FaultFlags And FAULT_OPEN) Then
            Msg = "  Driver failed to open"
        ElseIf (FaultFlags And FAULT_REG) Then
            Msg = "  Can't register board"
        ElseIf (FaultFlags And FAULT_SIG) Then
            Msg = "  Bad board signature"
        ElseIf (FaultFlags And FAULT_LED) Then
            Msg = "  Fault LED is stuck on"
        ElseIf (FaultFlags And FAULT_CRMT) Then
            Msg = "  CRMT failed to assert"
        ElseIf (FaultFlags And FAULT_DAV) Then
            Msg = "  DAV failed to assert"
        Else
            Msg = "  unknown fault type(s)"
        End If
        FaultMessage = "BOARD " & Format$(BdNum) & " FAULT(S)" & vbCrLf & Msg & vbCrLf
    Else
        FaultMessage = ""
    End If
End Function

Public Function SensorData(BdNum As Long) As String
    ' Construct a string that lists some sensor data.
    Dim ChNum   As Long     ' Channel number.
    Dim Msg     As String   ' Text string under construction.
    ' Abort if driver or board fault(s) detected.
    If DRV_GetFaultFlags(BdNum) Then
        Msg = FaultMessage(BdNum)
    Else
        ' Display heading.
        Msg = "BOARD " & Format$(BdNum) & ":" & vbCrLf
        ' Fetch and display sensor data from the first three channels.
        For ChNum = 0 To 2
            Msg = Msg & "  Channel " & Format$(ChNum) & ": " & Format$(DRV_GetSensorData(BdNum, ChNum)) & vbCrLf
        Next ChNum
        ' Fetch and display ambient temperature from the first termination board.
        Msg = Msg & "  Ambient 0:  " & Format$(DRV_GetAmbientTemp(BdNum, 0)) & vbCrLf
    End If
    SensorData = Msg
End Function

Public Function BoardAttr(BdNum As Long) As String
    ' Construct a string that lists the attributes of the specified board.
    Dim ProductID   As Long         ' Product model number.
    Dim Address     As Long         ' Board address.
    Dim Msg         As String       ' Attributes string under construction.
    Dim DotBuf      As String * 20  ' Buffer to receive network IP "dot" address.
    Dim DotAdrs     As String       ' Network IP "dot" address.
    ' Abort if board has any faults pending.
    If DRV_GetFaultFlags(BdNum) Then
        Msg = FaultMessage(BdNum)
    Else
        ' Fetch board's model number and address.
        ProductID = DRV_GetProductID(BdNum)
        Address = DRV_GetBoardAdrs(BdNum)
        ' Begin attributes string with header, model number and firmware version number.
        Msg = "BOARD " & Format$(BdNum) & " ATTRIBUTES:" & vbCrLf _
            & "  Model Number: " & Format$(ProductID) & vbCrLf _
            & "  FW Version:   " & Format$(DRV_GetFWVersion(BdNum) / 100, "#.00") & vbCrLf
        ' Append board's address to string; format depends on product family.
        Select Case ProductID
        Case 618 To 719 ' PCI ------------------------------------------------
            Msg = Msg & "  Bus Number:   " & Format$((Address \ 65536) And &HFFFF&) & vbCrLf _
                      & "  Slot Number:  " & Format$(Address And &HFFFF&) & vbCrLf
        Case 2518, 2519 ' ETHERNET -------------------------------------------
            DRV_InetNtoA DotBuf, Address   ' Convert binary IP address to "dot" format.
            DotAdrs = Left$(DotBuf, InStr(DotBuf, Chr$(0)) - 1)
            Msg = Msg & "  IP Address:   " & DotAdrs & vbCrLf
        Case Else  ' ISA, PC104, STD, VME ------------------------------------
            Msg = Msg & "  I/O BasePort: " & Hex$(Address) & vbCrLf
        End Select
    End If
    ' Return attributes string.
    BoardAttr = Msg
End Function

Public Sub ConfigChannels(BdNum As Long)
    ' Shows how to configure some channels for various sensor types.
    If DRV_GetFaultFlags(BdNum) Then Exit Sub   ' Abort if faults pending.
    DRV_SetSensorType BdNum, 0, TYP_VOLTS_5V    ' Chan 0 = 5V range.
    DRV_SetSensorType BdNum, 1, TYP_OHMS_400    ' Chan 1 = 400 ohm range.
    DRV_SetSensorType BdNum, 2, TYP_TC_K        ' Chan 2 = K thermocouple.
End Sub

Public Sub ResetAll(Lbl As Label, NUMBOARDS As Long)
    ' Reset all boards, display their product attributes and initialize channels.
    Dim BdNum As Long
    With Lbl
        .Caption = "RESET" & vbCrLf: .Refresh       ' Display heading.
        For BdNum = 0 To NUMBOARDS - 1              ' For each board ...
            DRV_ResetBoard BdNum                    '   Invoke soft reset.
            .Caption = .Caption & BoardAttr(BdNum)  '     Display board address and version.
            .Refresh
            ConfigChannels BdNum                    '     Declare some sensor types.
        Next BdNum
    End With
End Sub

Public Sub ShowSensorData(Lbl As Label, NUMBOARDS As Long)
    ' Fetch and display some sensor data from all boards.
    Dim BdNum   As Long
    With Lbl
        .Caption = "SENSOR DATA" & vbCrLf: .Refresh ' Display heading.
        For BdNum = 0 To NUMBOARDS - 1
            .Caption = .Caption & SensorData(BdNum) ' Display sensor data from all boards.
        Next BdNum
    End With
End Sub



